﻿/* _
MIT License

Copyright (c) 2022 zgock

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE. 
*/

using System;
using System.Collections.Generic;
using UnityEngine;

namespace PluginBuilder_Zgock
{
    class MCMApplyer : MVRScript
    {
        class MorphPair
        {
            public DAZMorph src;
            public DAZMorph dst;
            public DAZMorph shape;
        }
        private List<MorphPair> _pairlist = new List<MorphPair>();

        private Atom _person;

        private DAZCharacterSelector _characterSelector = null;

        public override void Init()
        {
            try
            {
                if (containingAtom.type != "Person")
                {
                    logError($"This plugin is for use with 'Person' atom only, not '{containingAtom.type}'");
                    return;
                }

                _person = containingAtom;

                _characterSelector = _person.GetComponentInChildren<DAZCharacterSelector>();
                if (_characterSelector == null)
                {
                    logError($"DAZCharacterSelector not found in '{_person.name}'");
                    return;
                }
                ScanMCM();
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        private void ScanMCM()
        {
            if (_characterSelector == null) return;
            _pairlist.Clear();
            DAZMorphBank bank = _characterSelector.morphBank1;
            foreach (DAZMorph morph in bank.morphs)
            {
                if (morph.visible && morph.displayName.Contains("_MCM_"))
                {
                    logDebug($"MCM [{morph.displayName}] found");
                    int index = morph.displayName.IndexOf("_MCM_");
                    string src = morph.displayName.Substring(0, index);
                    string shape = morph.displayName.Substring(index + "_MCM_".Length);
                    logDebug($"src = [{src}]");
                    logDebug($"shape = [{shape}]");

                    MorphPair pair = new MorphPair();
                    pair.dst = morph;
                    pair.src = bank.GetMorphByDisplayName(src);
                    pair.shape = bank.GetMorphByDisplayName(shape);

                    if (pair.src != null && pair.dst != null && pair.shape != null)
                    {
                        _pairlist.Add(pair);
                        logDebug("Add to pair list");
                    }
                    else
                    {
                        logDebug("Target not found");
                    }
                }
            }
        }

        void Update()
        {
            if (_characterSelector == null) return;
            foreach(MorphPair p in _pairlist)
            {
                float shapeval = p.shape.morphValue;
                float srcval = p.src.morphValue;
                p.dst.morphValue = shapeval * srcval;
            }
        }

        private void logDebug(string debugText)
        {
            //SuperController.LogMessage("MCMApplyer - " + debugText);
        }

        private void logMessage(string debugText)
        {
            SuperController.LogMessage("MCMApplyer - " + debugText);
        }

        private void logError(string debugText)
        {
            SuperController.LogError("MCMApplyer - " + debugText);
        }
    }
}
